<?php

namespace App\Notifications\SupportAgent;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use NotificationChannels\Telegram\TelegramMessage;
class OtherAgentsRepliedToTicket extends Notification
{
    use Queueable;

    /**
     * Create a new notification instance.
     *
     * @return void
     */
    public $ticket;
    public $comment;
    public $system;
    public function __construct($ticket, $comment, $system)
    {
        $this->ticket = $ticket;
        $this->comment = $comment;
        $this->system = $system;
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function via($notifiable)
    {
        $channels = [];
        if ($this->system['other_agents_replied_to_ticket_app_notif']) {
            $channels[] = 'database';
        }

        if ($this->system['other_agents_replied_to_ticket_mail_notif']) {
            $channels[] = 'mail';
        }

        if (
            !empty(config('constants.TELEGRAM_BOT_TOKEN')) &&
            !empty($notifiable->telegram_user_id) &&
            $this->system['other_agents_replied_to_ticket_telegram_notif']
        ) {
            $channels[] = 'telegram';
        }

        return $channels;
    }

    /**
     * Get the mail representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function toMail($notifiable)
    {   
        return (new MailMessage)
            ->subject('New reply on ticket '.$this->ticket['ticket_ref'])
            ->greeting('Hello '.$notifiable->name)
            ->line('You have received a new reply on ticket '.$this->ticket['ticket_ref'].' by  '.$this->comment->user['name'])
            ->line('<br><br><strong>'.$this->ticket['subject'].'</strong><br>')
            ->line($this->comment['comment'])
            ->action('Login to reply the comment', url('/login'));
    }

    /**
     * Get the array representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function toArray($notifiable)
    {
        return [
            'ticket_id' => $this->ticket['id'],
            'comment_id' => $this->comment['id'],
            'commented_by' => $this->comment['user_id']
        ];
    }

    public function toTelegram($notifiable)
    {
        if(
            !empty(config('constants.TELEGRAM_BOT_TOKEN')) &&
            !empty($notifiable->telegram_user_id)
        ) {

            $url = route('customer.view-ticket', ['id' => $this->ticket['id']]);

            $content = "*New reply on ticket ".$this->ticket['ticket_ref']."*\n\nYou have received a new reply on ticket ".$this->ticket['ticket_ref']." by  *".$this->comment->user['name']."*\n\n*Subject:* ".trim($this->ticket['subject'])."\n\n*Comment:* ".strip_tags($this->comment['comment']);

            return TelegramMessage::create()
                ->to($notifiable->telegram_user_id)
                ->content(html_entity_decode($content))
                ->button('View Ticket', $url);
        }
    }
}
