<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

class TicketComment extends Model
{
    /**
	 * The attributes that aren't mass assignable.
	 *
	 * @var array
	 */
	protected $guarded = ['id'];

    protected $appends = [
        'media_urls'
    ];

    /**
     * All of the relationships to be touched.
     *
     * @var array
     */
    protected $touches = ['ticket'];

	/**
     * Get the ticket that owns the comment.
     */
    public function ticket()
    {
        return $this->belongsTo('App\Ticket');
    }

    /**
     * Get the user that owns the comment.
     */
    public function user()
    {
        return $this->belongsTo('App\User');
    }

    /**
     * Get the comment's media
     */
    public function medias()
    {
        return $this->morphMany('App\Media', 'mediable');
    }

    public function getMediaUrlsAttribute()
    {
        $medias = $this->medias;
        $urls = [];
        if(!empty($medias)){
            foreach ($medias as $index => $media) {
                if(!empty($media->name)) {
                    $name_arr = explode('_', $media->name, 2);
                    $name = !empty($name_arr[1]) ? $name_arr[1] : $name_arr[0];
                    $urls[] = [
                        'url' => asset('uploads/'.config('constants.media_folder').'/'.$media->name),
                        'name' => $name
                    ];
                }
            }
        }
        return $urls;
    }
}
